@@ -11,7 +11,7 @@ We then need to worry about multiple columns if there are. If none, each timeser
 If there are multiple columns, we will perform the calculations columns by columns. 
 
 """
-from pyg_base._types import is_df, is_str, is_num, is_tss, is_int, is_arr, is_ts, is_arrs, is_tuples, is_pd, is_date
+from pyg_base._types import is_df, is_str, is_num, is_series, is_tss, is_int, is_arr, is_ts, is_arrs, is_tuples, is_pd, is_date
 from pyg_base._dictable import dictable
 from pyg_base._as_list import as_list
 from pyg_base._zip import zipper
@@ -273,8 +273,8 @@ def _nona(df, value = np.nan):
         mask = np.isinf(df)
     else:
         mask = df == value
-    if len(mask.shape) == 2:
-        mask = mask.min(axis=1) == 1
+    while len(mask.shape) > 1:
+        mask = mask.min(axis = 1)
     return df[~mask]
 
 def nona(a, value = np.nan):
@@ -320,7 +320,7 @@ def _df_reindex(ts, index, method = None, limit = None):
         return res
     elif is_arr(ts):
         if isinstance(index, pd.Index):
-            if len(index) == len(ts):
+            if len(index) == len(ts) or len(ts) <= 1:
                 return ts
             else:
                 raise ValueError('trying to reindex numpy array %s using pandas index %s'%(ts, index))
@@ -437,7 +437,7 @@ def df_reindex(ts, index = None, method = None, limit = None):
     return _df_reindex(ts, index = index, method = method, limit = limit)
 
 
-def df_concat(objs, columns = None, axis=1, join = 'outer'):
+def df_concat(objs, columns = None, axis = 1, join = 'outer', method = None, limit = None):
     """
     simple concatenator, 
     - defaults to to concatenating by date (for timeseries)
@@ -521,6 +521,7 @@ def df_concat(objs, columns = None, axis=1, join = 'outer'):
             res.columns = columns 
         else:
             res = res.rename(columns = columns)
+    res = df_fillna(res, method = method, axis = 1 if axis == 0 else 1, limit = limit)
     return res
 
 
@@ -680,7 +681,7 @@ def df_sync(dfs, join = 'ij', method = None, columns = 'ij'):
     else:
         return dfs
     listed = _list(values)
-    tss = [ts for ts in listed if is_ts(ts)]
+    tss = [ts for ts in listed if is_df(ts)]
     index = df_index(listed, join)
     dfs = df_reindex(dfs, index, method = method)
 
@@ -691,7 +692,35 @@ def df_sync(dfs, join = 'ij', method = None, columns = 'ij'):
         cols = df_columns(tss, columns)
         dfs = df_recolumn(dfs, cols)
     return dfs
+
+def _mask(df, value):
+    if is_num(value) and np.isnan(value):
+        return np.isnan(df)
+    elif is_num(value):
+        return df == value
+    elif isinstance(value, list):
+        return reducer(np.maximum, [_mask(df, v) for v in value]) 
+    else:
+        raise ValueError('not sure how to construct a mask from %s'%value)
+
+def df_apply(df, func, axis = 0, raw = False, result_type = None, args = None, params = None, exc = np.nan):
+    """
+    similar to df.apply(func, axis...) except we nan results when we don't have any entries in original calculation
+
+    :Example:
+    ---------
+    >>> df = pd.DataFrame(dict(a = [np.nan, np.nan], b = [np.nan, 1.]))
+    >>> assert np.isnan(df_apply(df, 'sum').a)
+    >>> assert df.apply('sum').a == 0
     
+    """
+    params = params or {}
+    args = args or ()
+    res = df.apply(func, axis = axis, raw = False, result_type = None, args = args, **params)
+    if len(as_list(exc)) > 0:
+        mask = ~_mask(df, exc)
+        res[mask.sum(axis = axis) == 0] = np.nan
+    return res   
 
 class presync(wrapper):
     """
@@ -864,7 +893,12 @@ class presync(wrapper):
                 if is_int(columns):
                     res = {i: self.function(*df_column(args_, column = None, i = i), **df_column(kwargs_, column=None, i = i)) for i in range(columns)}
                 elif columns is None:
-                    return self.function(*df_column(args_, column = None), **df_column(kwargs_, column = None))
+                    res = self.function(*df_column(args_, column = None), **df_column(kwargs_, column = None))
+                    if isinstance(res, pd.Series):
+                        cols1 = [tuple(ts.columns) for ts in tss if is_df(ts) and ts.shape[1]==1]
+                        if len(set(cols1))>0:
+                            res = pd.DataFrame(res) 
+                    return res
                 else:
                     columns = list(columns) if isinstance(columns, pd.Index) else as_list(columns)
                     columns = sorted(columns)
@@ -878,7 +912,12 @@ def _div_(a, b):
     """
     division of a by b supporting presynching (inner join) of timeseries
     """
-    return a/b
+    if is_num(b):
+        return np.nan if b == 0 else a/b
+    else:
+        denom = b.copy()
+        denom[denom == 0] = np.nan
+        return a/denom
 
 @presync
 def _sub_(a, b):
@@ -1013,13 +1052,32 @@ def le_(a, b, join = 'ij', method = None, columns = 'ij'):
     """
     return _le_(a,b, join = join, method = method, columns = columns)
 
+def _align_columns(a, b, func):
+    da, db = [len(getattr(x, 'shape', ())) for x in (a,b)]
+    if da == db or max(da, db) <= 1:
+        return func(a,b)
+    a, b = (a, b) if da == 2 else (b, a)
+    if is_series(b):
+        b = pd.concat([b] * a.shape[1], axis=1)
+        b.columns = a.columns
+    else:
+        b = np.concatenate([b] * a.shape[1], axis=1)
+    return func(a,b)
+        
+def _minimum(a, b):
+    return _align_columns(a, b, np.minimum)
+
+def _maximum(a, b):
+    return _align_columns(a, b, np.maximum)
+
+
 def min_(a, b = None, join = 'ij', method = None, columns = 'ij'):
     """
     equivalent to redced np.minimum operation supporting presynching of timeseries
     """
     dfs = as_list(a) + as_list(b)
-    dfs = df_sync(dfs, join = join, method = method, columns = columns)    
-    return reducer(np.minimum, dfs)
+    dfs = df_sync(dfs, join = join, method = method, columns = columns)
+    return reducer(_minimum, dfs)
 
 def max_(a, b = None, join = 'ij', method = None, columns = 'ij'):
     """
@@ -1027,7 +1085,119 @@ def max_(a, b = None, join = 'ij', method = None, columns = 'ij'):
     """
     dfs = as_list(a) + as_list(b)
     dfs = df_sync(dfs, join = join, method = method, columns = columns)
-    return reducer(np.maximum, dfs)
+    return reducer(_maximum, dfs)
+
+def mask2v(df, mask = np.nan, value = 0.0):
+    if not is_pd(mask):
+        mask = _mask(df, mask)
+    res = df.copy()
+    res[mask] = value
+    return res
+
+def df_count(a, b = None, join = 'oj', method = None, columns = 'oj', exc = np.nan):
+    """
+    apply count() on multiple dataframes
+    - synch the data
+    - removes nans (or any value in mask)
+
+    :Example:
+    ----------
+    >>> a = pd.DataFrame(dict(a = [1, 2], b = [np.nan, np.nan], c = [1, 2.]))
+    >>> b = pd.DataFrame(dict(a = [np.nan,2], b = [2, np.nan]))
+    >>> df_count(a,b, columns = 'oj')
+    
+       a  b  c
+    0  1  1  1
+    1  2  0  1
+    
+    """
+    dfs = as_list(a) + as_list(b)
+    dfs = df_sync(dfs, join = join, method = method, columns = columns)
+    masks = [_mask(df, exc) for df in dfs]
+    n = sum([~mask for mask in masks])
+    return n
+
+
+def df_sum(a, b = None, join = 'oj', method = None, columns = 'oj', exc = np.nan):
+    """
+    apply sum() on multiple dataframes
+    - synch the data
+    - removes nans (or any value in mask)
+    
+    :Example:
+    ----------
+    >>> a = pd.DataFrame(dict(a = [1, 2], b = [np.nan, np.nan], c = [1, 2.]))
+    >>> b = pd.DataFrame(dict(a = [np.nan,2], b = [2, np.nan]))
+    >>> df_sum(a,b)
+    
+         a    b    c
+    0  1.0  2.0  1.0
+    1  4.0  NaN  2.0
+    
+    """
+    dfs = as_list(a) + as_list(b)
+    dfs = df_sync(dfs, join = join, method = method, columns = columns)
+    masks = [_mask(df, exc) for df in dfs]
+    clean_dfs = [mask2v(df, mask, 0.0) for df, mask in zip(dfs, masks)]
+    res = sum(clean_dfs)
+    n = sum([~mask for mask in masks])
+    res[n == 0] = np.nan
+    return res       
+        
+def df_mean(a, b = None, join = 'oj', method = None, columns = 'oj', exc = np.nan):
+    """
+    apply mean() on multiple dataframes
+    - synch the data
+    - removes nans (or any value in mask)
+    
+    :Example:
+    ----------
+    >>> a = pd.DataFrame(dict(a = [1, 2], b = [np.nan, np.nan], c = [1, 2.]))
+    >>> b = pd.DataFrame(dict(a = [np.nan,2], b = [2, np.nan]))
+    >>> df_mean(a,b)
+    
+         a    b    c
+    0  1.0  2.0  1.0
+    1  2.0  NaN  2.0
+
+    """
+    dfs = as_list(a) + as_list(b)
+    dfs = df_sync(dfs, join = join, method = method, columns = columns)
+    masks = [_mask(df, exc) for df in dfs]
+    clean_dfs = [mask2v(df, mask, 0.0) for df, mask in zip(dfs, masks)]
+    res = sum(clean_dfs)
+    n = sum([~mask for mask in masks])
+    n[n == 0] = np.nan
+    return res / n
+
+def df_std(a, b = None, join = 'oj', method = None, columns = 'oj', exc = np.nan):
+    """
+    apply biased std() on multiple dataframes
+    - synch the data
+    - removes nans (or any value in mask)
+    
+    :Example:
+    ----------
+    >>> a = pd.DataFrame(dict(a = [1, 2], b = [np.nan, np.nan], c = [1, 2.]))
+    >>> b = pd.DataFrame(dict(a = [np.nan,2], b = [2, np.nan]))
+    >>> df_std(a,b)
+    
+         a    b    c
+    0  1.0  2.0  1.0
+    1  2.0  NaN  2.0
+
+    """
+    dfs = as_list(a) + as_list(b)
+    dfs = df_sync(dfs, join = join, method = method, columns = columns)
+    masks = [_mask(df, exc) for df in dfs]
+    clean_dfs = [mask2v(df, mask, 0.0) for df, mask in zip(dfs, masks)]
+    m1 = sum(clean_dfs)
+    m2 = sum([df**2 for df in clean_dfs])
+    n = sum([~mask for mask in masks])
+    n[n < 2] = np.nan
+    return (m2/n - (m1/n)**2) ** 0.5
+
+
 
 
 def _closed(oc):
@@ -1153,8 +1323,12 @@ def df_slice(df, lb = None, ub = None, openclose = '(]', n = 1):
         return pd.concat([pre, post]).sort_index()        
     if isinstance(df, list): 
         if isinstance(lb, list) and ub is None:
+            lb2df = dict(zip(lb,df))
+            lb = sorted(lb); df = [lb2df[key] for key in lb]
             ub = lb[1:] + [None]
         elif isinstance(ub, list) and lb is None:
+            ub2df = dict(zip(ub,df))
+            ub = sorted(ub); df = [ub2df[key] for key in ub]
             lb = [None] + ub[:-1]
         boundaries = sorted(set([date for date in lb + ub if date is not None]))
         df = [d if is_pd(d) else pd.Series(d, boundaries) for d in df]
